unit DemoMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ExtCtrls, StdCtrls, ComCtrls, TypInfo;

type
  TDemoMainForm = class(TForm)
    MainMenu1: TMainMenu;
    Help1: TMenuItem;
    About1: TMenuItem;
    Panel1: TPanel;
    Splitter1: TSplitter;
    Panel2: TPanel;
    Panel3: TPanel;
    Panel4: TPanel;
    Label1: TLabel;
    UnitLabel: TLabel;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    PropListBox: TListBox;
    EventsListBox: TListBox;
    ClassesTree: TTreeView;
    procedure About1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ClassesTreeChange(Sender: TObject; Node: TTreeNode);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  DemoMainForm: TDemoMainForm;

implementation

uses AboutFrm, GblClass;

{$R *.DFM}

procedure TDemoMainForm.About1Click(Sender: TObject);
begin
  with TAbout.Create(Application) do
    try
      ShowModal
    finally
      Free
    end  
end;

procedure TDemoMainForm.FormCreate(Sender: TObject);
var
  ClassList: TList;

  function InsertClass(AClass: TClass): TTreeNode;
  var
    I: Integer;
    ParentNode: TTreeNode;
  begin
    ParentNode := nil;
    if AClass.ClassParent <> nil then
      if ClassList.IndexOf(AClass.ClassParent) = -1 then
      begin
        for I := 0 to ClassesTree.Items.Count - 1 do
          if ClassesTree.Items[I].Data = AClass.ClassParent then
          begin
            ParentNode := ClassesTree.Items[I];
            Break
          end
      end
      else ParentNode := InsertClass(AClass.ClassParent);
    Result := ClassesTree.Items.AddChildObject(ParentNode, AClass.ClassName, AClass);
    ClassList.Remove(AClass)
  end;

var
  I: Integer;
begin
  ClassList := TList.Create;
  try
    for I := 0 to GlobalClassCount - 1 do ClassList.Add(GlobalClass(I));
    ClassesTree.Items.BeginUpdate;
    try
      while ClassList.Count > 0 do InsertClass(ClassList[0]);
      ClassesTree.AlphaSort
    finally
      ClassesTree.Items.EndUpdate
    end
  finally
    ClassList.Free
  end
end;

procedure TDemoMainForm.ClassesTreeChange(Sender: TObject;
  Node: TTreeNode);
var
  TypeData: PTypeData;
  Props: PPropList;
  I: Integer;
begin
  PropListBox.Clear;
  EventsListBox.Clear;
  UnitLabel.Caption := '';
  if (Node = nil) or (Node.Data = nil) or
    (TClass(Node.Data).ClassInfo = nil) then Exit;
  TypeData := GetTypeData(TClass(Node.Data).ClassInfo);
  if TypeData = nil then Exit;
  UnitLabel.Caption := 'Unit - ' + TypeData^.UnitName;
  if TypeData^.PropCount = 0 then Exit;
  GetMem(Props, TypeData^.PropCount * sizeof(Pointer));
  try
    GetPropInfos(TClass(Node.Data).ClassInfo, Props);
    for I := 0 to TypeData^.PropCount - 1 do
      with Props^[I]^ do
        if PropType^^.Kind = tkMethod then
          EventsListBox.Items.Add(Name + ': ' + PropType^^.Name)
        else
          PropListBox.Items.Add(Name + ': ' + PropType^^.Name)
  finally
    FreeMem(Props)
  end
end;

end.
